/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.queries;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Resource;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.jdbc.DatabaseConstants;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

public class SQLQueryProcessor implements QueryProcessor {

    private static final Log log = LogFactory.getLog(SQLQueryProcessor.class);

    public Resource executeQuery(Resource query, Object[] parameters, Connection conn)
            throws RegistryException {

        try {

            String sqlString = (String) query.getContent();

            PreparedStatement s = conn.prepareStatement(sqlString);

            if (parameters != null) {
                for (int i = 0; i < parameters.length; i++) {
                    s.setObject(i + 1, parameters[i]);
                }
            }

            ResultSet results = s.executeQuery();

            String resultType =
                    (String) query.getProperties().get(RegistryConstants.RESULT_TYPE_PROPERTY_NAME);

            if (resultType == null || resultType.equals(RegistryConstants.RESOURCES_RESULT_TYPE)) {

                // Result is a normal resource, which is stored in the Resources table or a collection
                // of normal resources.

                Resource resourcesCollection = fillResourcesCollection(results, conn);
                return resourcesCollection;

            } else if (resultType.equals(RegistryConstants.COMMENTS_RESULT_TYPE)) {

                Resource commentsCollection = fillCommentsCollection(results, conn);
                return commentsCollection;

            }  else if (resultType.equals("Tags")) {

                Resource tagsCollection = fillTagsCollection(results, conn);
                return tagsCollection;

            } else if (resultType.equals("Ratings")) {

                // URL for a rating /p1/r1?rating&user=xyz
            
            } else if (resultType.equals("TagsWithCount")) {

                // Result is in the form of (tag, number of taggings) tuples
                // Fill TagCount objects with results and add them as an array



            } else if (resultType.equals("ResourcePathsWithTagCount")) {

                // Result is in the form of (resource path, number of taggings) tuples
                // Fill ResourceTagCount objects with results and add them as an array

            }

            String msg = "Unknown result type: " + resultType + " defined for the query: " +
                    sqlString + " located in path: " + query.getPath();
            log.error(msg);
            throw new RegistryException(msg);

        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    private Resource fillResourcesCollection(ResultSet results, Connection conn)
            throws SQLException {

        // Result is a normal resource, which is stored in the Resources table or a collection
        // of normal resources.

        List pathList = new ArrayList();

        while (results.next()) {
            String path = results.getString(DatabaseConstants.PATH_FIELD);
            pathList.add(path);
        }

        String[] paths = (String[]) pathList.toArray(new String[pathList.size()]);

        Resource resourcesCollection = new Resource();
        resourcesCollection.setDirectory(true);
        resourcesCollection.setContent(paths);

        return resourcesCollection;
    }

    private Resource fillCommentsCollection(ResultSet results, Connection conn)
            throws SQLException {

        // SQL query should resturn a list of DatabaseConstants.COMMENT_ID_FIELDs.
        // resultArtifact contains a String[] of URLs for comments.

        // URL for a comment <resource_path>?comment<comment_id>
        // e.g. /p1/r1?comment12

        List commentPathList = new ArrayList();

        while (results.next()) {

            long commentID = results.getLong(DatabaseConstants.COMMENT_ID_FIELD);

            String selectResourcePath =
                    "SELECT A.PATH FROM ARTIFACTS A, COMMENTS C WHERE A.AID=C.AID AND C.CM_ID=?";
            PreparedStatement s1 = conn.prepareStatement(selectResourcePath);
            s1.setLong(1, commentID);

            String path = "";
            ResultSet result = s1.executeQuery();
            if (result.next()) {
                path = result.getString(DatabaseConstants.PATH_FIELD);
            }

            String commentPath = path + "?comment" + commentID;
            commentPathList.add(commentPath);
        }

        String[] commentPaths =
                (String[]) commentPathList.toArray(new String[commentPathList.size()]);

        Resource commentsCollection = new Resource();
        commentsCollection.setDirectory(true);
        commentsCollection.setContent(commentPaths);

        return commentsCollection;
    }

    private Resource fillTagsCollection(ResultSet results, Connection conn) throws SQLException {

        // URL for a tag /p1/r1?tag=java&user=xyz

        List tagPathList = new ArrayList();
        while (results.next()) {

            String userName = results.getString(DatabaseConstants.USER_ID_FIELD);
            long artifactID = results.getLong(DatabaseConstants.ARTIFACT_ID_FIELD);
            long tagID = results.getLong(DatabaseConstants.TAG_ID_FIELD);

            String artifactSQL = "SELECT PATH FROM ARTIFACTS WHERE AID=?";
            PreparedStatement s1 = conn.prepareStatement(artifactSQL);
            s1.setLong(1, artifactID);
            ResultSet result1 = s1.executeQuery();

            String taggedPath = "";
            if (result1.next()) {
                taggedPath = result1.getString(DatabaseConstants.PATH_FIELD);
            }

            String tagSQL = "SELECT TAG_NAME FROM TAGS WHERE TAG_ID=?";
            PreparedStatement s2 = conn.prepareStatement(tagSQL);
            s2.setLong(1, tagID);
            ResultSet result2 = s2.executeQuery();

            String tag = "";
            if (result2.next()) {
                tag = result2.getString(DatabaseConstants.TAG_NAME_FIELD);
            }

            String tagPath = taggedPath + "?tag=" + tag + "&user=" + userName;
            tagPathList.add(tagPath);
        }

        String[] tagPaths = (String[]) tagPathList.toArray(new String[tagPathList.size()]);

        Resource tagCollection = new Resource();
        tagCollection.setDirectory(true);
        tagCollection.setContent(tagPaths);

        return tagCollection;
    }
}
